<?php
/*
Plugin Name: WP Include File Source
Plugin URI: http://www.atwillys.de/repository/include-source
Description: Pastes the highlighted contents of a source file in your post if you type [include-source file=</a file related to server document root>]. Alternatively multiple files file1="..." file2="..." and types (e.g. [include-source type="php" file="..."]). The WP Include-File-Source uses the plugin "syntaxhighlighter".
Version: 1.0
Author: Stefan Wilhelm
Author URI: http://www.atwillys.de
*/

class WP_Plugin_IncludeSource {
	
	public function __construct() {
		register_activation_hook( __FILE__, array($this, 'onActivate' ));
		register_deactivation_hook( __FILE__, array($this, 'onDeactivate' ));
		add_filter( 'the_content', array($this, 'onFilter'), 6);
	}
	
	public function onActivate() {
	}

	public function onDeactivate() {
	}

	public function onFilter($content) {
		global $shortcode_tags;
		$orig_shortcode_tags = $shortcode_tags;
		remove_all_shortcodes();
		add_shortcode('include-source', array($this, 'onShortCode_includeSource'));
		$content = do_shortcode($content);
		$shortcode_tags = $orig_shortcode_tags;
		return $content;
	}

	public function onShortCode_includeSource($atts, $content=null) {
		extract(shortcode_atts(array('file' => '', 'lang' => '', 'type' => '', 'file1' => '', 'file2' => '', 'file3' => '', 'file4' => '', 'file5' => ''), $atts));

		$lang = trim(str_replace('"', '', $lang));
		$type = trim(str_replace('"', '', $type));
		if(!empty($lang)) $type = $lang; unset($lang);

		$files = array();
		if(!empty($file)) $files[]  = trim(str_replace('"', '',$file), '/ '); unset($file);
		if(!empty($file1)) $files[] = trim(str_replace('"', '',$file1), '/ '); unset($file1);
		if(!empty($file2)) $files[] = trim(str_replace('"', '',$file2), '/ '); unset($file2);
		if(!empty($file3)) $files[] = trim(str_replace('"', '',$file3), '/ '); unset($file3);
		if(!empty($file4)) $files[] = trim(str_replace('"', '',$file4), '/ '); unset($file4);
		if(!empty($file5)) $files[] = trim(str_replace('"', '',$file5), '/ '); unset($file5);

		if(empty($files)) {
			$content = '<b>Read file: No file specified, use <i>include-source file="..." type="{php,cpp ...}"</i></b>';
		} else {
			
			if(empty($type)) {
				$extension = explode('.', end(explode('/', reset($files))));
				if(count($extension) > 1) {		
					$extension = strtolower(end($extension));
					$keys = array(
						'cc' => 'cpp',
						'hpp' => 'cpp',
					);
					$type = isset($keys[$extension]) ? $keys[$extension] : $extension;
				}
			}

			foreach($files as $key => $file) {
				$file = rtrim($_SERVER['DOCUMENT_ROOT'], '/ ') . "/$file";
				if (!is_file($file)) {
					$files[$key] = "<b>Read file: File does not exist: $file</b>";
				} else if (!is_readable($file)) {
					$files[$key] = "<b>Read file: File not readable: $file</b>";
				} else {
					ob_start();
					$files[$key] = trim(@file_get_contents($file));
					ob_get_clean();
				}
			}
			$content = implode(empty($type) ? '<br />' : "\n\n", $files);
		}

		if(!empty($type)) {
			$content = '[sourcecode lang="' . $type . '"]' . "\n$content\n" . '[/sourcecode]';
		} else {
			$content = htmlspecialchars($content);
		}
	
		return $content;
	}
}

function WP_Plugin_IncludeSource_Startup() {
	global $wp_plugin_includesource;
	$wp_plugin_includesource = new WP_Plugin_IncludeSource();
}
add_action( 'init', 'WP_Plugin_IncludeSource_Startup', 4);
?>